import { NO_ERRORS_SCHEMA } from '@angular/core';
import { async, TestBed } from '@angular/core/testing';
import { FormsModule } from '@angular/forms';
import { Broadcaster } from 'ngx-base';
import { Che } from '../services/che';
import { CodebasesItemHeadingComponent } from './codebases-item-heading.component';

describe('Codebases Item Heading Component', () => {
  let broadcasterMock: any;
  let fixture;
  let element: HTMLElement;

  beforeEach(() => {
    broadcasterMock = jasmine.createSpyObj('Broadcaster', ['on']);

    TestBed.configureTestingModule({
      imports: [FormsModule],
      declarations: [CodebasesItemHeadingComponent],
      providers: [
        {
          provide: Broadcaster,
          useValue: broadcasterMock,
        },
      ],
      // Tells the compiler not to error on unknown elements and attributes
      schemas: [NO_ERRORS_SCHEMA],
    });
    fixture = TestBed.createComponent(CodebasesItemHeadingComponent);
    fixture.detectChanges();
    element = fixture.nativeElement;
  });

  it('Che starting message', async(() => {
    const comp = fixture.componentInstance;
    comp.cheState = { running: false } as Che;
    fixture.detectChanges();

    expect(comp.getNotificationMessage()).toEqual(comp.cheStartingMessage);
  }));

  it('Che running message', async(() => {
    const comp = fixture.componentInstance;
    comp.cheState = { running: true } as Che;
    fixture.detectChanges();

    expect(comp.getNotificationMessage()).toEqual(comp.cheRunningMessage);
  }));

  it('Che performing multi-tenant migration message', async(() => {
    const comp = fixture.componentInstance;
    comp.cheState = { running: false, multiTenant: true } as Che;
    fixture.detectChanges();

    expect(comp.getNotificationMessage()).toEqual(comp.chePerformingMultiTenantMigrationMessage);
  }));

  it('Che finished multi-tenant migration message', async(() => {
    const comp = fixture.componentInstance;
    comp.cheState = { running: true, multiTenant: true } as Che;
    fixture.detectChanges();

    expect(comp.getNotificationMessage()).toEqual(comp.cheFinishedMultiTenantMigrationMessage);
  }));

  it('Show Security alert in heading', async(() => {
    const comp = fixture.componentInstance;
    comp.cveNotify = true;
    fixture.detectChanges();

    const cveElementTag = element.querySelector('.security-alert-heading');
    expect(cveElementTag).toBeDefined();
  }));

  it('Should not show Security alert in heading', async(() => {
    const comp = fixture.componentInstance;
    comp.cveNotify = false;
    fixture.detectChanges();

    const cveElementTag = element.querySelector('.security-alert-heading');
    expect(cveElementTag).toBeNull();
  }));
});
