import {
  Component,
  EventEmitter,
  Input,
  OnChanges,
  OnInit,
  Output,
  SimpleChanges,
  TemplateRef,
  ViewChild,
  ViewEncapsulation,
} from '@angular/core';
import { FilterConfig, FilterEvent, FilterField } from 'patternfly-ng/filter';
import { SortConfig, SortEvent } from 'patternfly-ng/sort';
import { ToolbarConfig } from 'patternfly-ng/toolbar';

@Component({
  encapsulation: ViewEncapsulation.None,
  selector: 'areas-toolbar',
  templateUrl: './areas-toolbar.component.html',
  styleUrls: ['./areas-toolbar.component.less'],
})
export class AreasToolbarComponent implements OnChanges, OnInit {
  @Input() resultsCount: number = 0;

  @Input() userOwnsSpace: boolean;

  @Output('onAddArea') onAddArea = new EventEmitter();

  @Output('onFilterChange') onFilterChange = new EventEmitter();

  @Output('onSortChange') onSortChange = new EventEmitter();

  @ViewChild('addAreasTemplate') addAreasTemplate: TemplateRef<any>;

  filterConfig: FilterConfig;

  isAscendingSort: boolean = true;

  sortConfig: SortConfig;

  toolbarConfig: ToolbarConfig;

  constructor() {}

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.resultsCount && this.filterConfig) {
      this.filterConfig.resultsCount = changes.resultsCount.currentValue;
    }
  }

  // Initialization

  ngOnInit(): void {
    this.filterConfig = {
      fields: [
        {
          id: 'area',
          title: 'Area',
          placeholder: 'Filter by Area...',
          type: 'text',
        },
      ] as FilterField[],
      appliedFilters: [],
      resultsCount: 0,
      selectedCount: 0,
      totalCount: 0,
    } as FilterConfig;

    this.sortConfig = {
      fields: [
        {
          id: 'area',
          title: 'Area',
          sortType: 'alpha',
        },
      ],
      isAscending: this.isAscendingSort,
    } as SortConfig;

    this.toolbarConfig = {
      filterConfig: this.filterConfig,
      sortConfig: this.sortConfig,
    } as ToolbarConfig;
  }

  // Actions

  addArea($event: MouseEvent): void {
    this.onAddArea.emit($event);
  }

  filterChange($event: FilterEvent): void {
    this.onFilterChange.emit($event);
  }

  sortChange($event: SortEvent): void {
    this.onSortChange.emit($event);
  }
}
